package org.springframework.boot.loader;

import java.io.IOException;
import java.net.URL;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.springframework.boot.loader.ClassPathIndexFile;
import org.springframework.boot.loader.ExecutableArchiveLauncher;
import org.springframework.boot.loader.archive.Archive;
import org.springframework.boot.loader.archive.ExplodedArchive;
import org.springframework.boot.loader.archive.Archive.Entry;
import org.springframework.boot.loader.archive.Archive.EntryFilter;

public class JarLauncher extends ExecutableArchiveLauncher {

   private static final String DEFAULT_CLASSPATH_INDEX_LOCATION = "BOOT-INF/classpath.idx";
   static final EntryFilter NESTED_ARCHIVE_ENTRY_FILTER = (param1) -> {
      return entry.isDirectory()?entry.getName().equals("BOOT-INF/classes/"):entry.getName().startsWith("BOOT-INF/lib/");
   };


   public JarLauncher() {
   }

   protected JarLauncher(Archive archive) {
      super(archive);
   }

   protected ClassPathIndexFile getClassPathIndex(Archive archive) throws IOException {
      if(archive instanceof ExplodedArchive) {
         String location = this.getClassPathIndexFileLocation(archive);
         return ClassPathIndexFile.loadIfPossible((URL)archive.getUrl(), (String)location);
      } else {
         return super.getClassPathIndex(archive);
      }
   }

   private String getClassPathIndexFileLocation(Archive archive) throws IOException {
      Manifest manifest = archive.getManifest();
      Attributes attributes = manifest != null?manifest.getMainAttributes():null;
      String location = attributes != null?attributes.getValue("Spring-Boot-Classpath-Index"):null;
      return location != null?location:"BOOT-INF/classpath.idx";
   }

   protected boolean isPostProcessingClassPathArchives() {
      return false;
   }

   protected boolean isSearchCandidate(Entry entry) {
      return entry.getName().startsWith("BOOT-INF/");
   }

   protected boolean isNestedArchive(Entry entry) {
      return NESTED_ARCHIVE_ENTRY_FILTER.matches(entry);
   }

   public static void main(String[] args) throws Exception {
      (new JarLauncher()).launch(args);
   }

}
